@@ -82,7 +82,7 @@ PhysicalDisk = collections.namedtuple(
     ['id', 'description', 'controller', 'manufacturer', 'model', 'media_type',
      'interface_type', 'size_mb', 'free_size_mb', 'serial_number',
      'firmware_version', 'status', 'raid_status', 'sas_address',
-     'device_protocol'])
+     'device_protocol', 'bus'])
 
 RAIDController = collections.namedtuple(
     'RAIDController', ['id', 'description', 'manufacturer', 'model',
@@ -95,12 +95,225 @@ VirtualDisk = collections.namedtuple(
      'status', 'raid_status', 'span_depth', 'span_length',
      'pending_operations', 'physical_disks'])
 
-
 NO_FOREIGN_DRIVES = ["STOR058", "STOR018"]
 
 
+class RAIDAttribute(object):
+    """Generic RAID attribute class"""
+
+    def __init__(self, name, instance_id, current_value, pending_value,
+                 read_only, fqdd):
+        """Creates RAIDAttribute object
+
+        :param name: name of the RAID attribute
+        :param instance_id: InstanceID of the RAID attribute
+        :param current_value: list containing the current values of the
+                RAID attribute
+        :param pending_value: pending value of the RAID attribute, reflecting
+                an unprocessed change (eg. config job not completed)
+        :param read_only: indicates whether this RAID attribute can be changed
+        :param fqdd: Fully Qualified Device Description of the RAID Attribute
+        """
+
+        self.name = name
+        self.instance_id = instance_id
+        self.current_value = current_value
+        self.pending_value = pending_value
+        self.read_only = read_only
+        self.fqdd = fqdd
+
+    def __eq__(self, other):
+        return self.__dict__ == other.__dict__
+
+    @classmethod
+    def parse(cls, namespace, raid_attr_xml):
+        """Parses XML and creates RAIDAttribute object"""
+
+        name = utils.get_wsman_resource_attr(
+            raid_attr_xml, namespace, 'AttributeName')
+        instance_id = utils.get_wsman_resource_attr(
+            raid_attr_xml, namespace, 'InstanceID')
+        current_value = [attr.text for attr in
+                         utils.find_xml(raid_attr_xml, 'CurrentValue',
+                                        namespace, find_all=True)]
+        pending_value = utils.get_wsman_resource_attr(
+            raid_attr_xml, namespace, 'PendingValue', nullable=True)
+        read_only = utils.get_wsman_resource_attr(
+            raid_attr_xml, namespace, 'IsReadOnly')
+        fqdd = utils.get_wsman_resource_attr(
+            raid_attr_xml, namespace, 'FQDD')
+
+        return cls(name, instance_id, current_value, pending_value,
+                   (read_only == 'true'), fqdd)
+
+
+class RAIDEnumerableAttribute(RAIDAttribute):
+    """Enumerable RAID attribute class"""
+
+    namespace = uris.DCIM_RAIDEnumeration
+
+    def __init__(self, name, instance_id, current_value, pending_value,
+                 read_only, fqdd, possible_values):
+        """Creates RAIDEnumerableAttribute object
+
+        :param name: name of the RAID attribute
+        :param instance_id: InstanceID of the RAID attribute
+        :param current_value: list containing the current values of the
+                RAID attribute
+        :param pending_value: pending value of the RAID attribute, reflecting
+                an unprocessed change (eg. config job not completed)
+        :param read_only: indicates whether this RAID attribute can be changed
+        :param fqdd: Fully Qualified Device Description of the RAID
+                Attribute
+        :param possible_values: list containing the allowed values for the RAID
+                attribute
+        """
+        super(RAIDEnumerableAttribute, self).__init__(name, instance_id,
+                                                      current_value,
+                                                      pending_value,
+                                                      read_only, fqdd)
+
+        self.possible_values = possible_values
+
+    @classmethod
+    def parse(cls, raid_attr_xml):
+        """Parses XML and creates RAIDEnumerableAttribute object"""
+
+        raid_attr = RAIDAttribute.parse(cls.namespace, raid_attr_xml)
+        possible_values = [attr.text for attr
+                           in utils.find_xml(raid_attr_xml,
+                                             'PossibleValues',
+                                             cls.namespace, find_all=True)]
+
+        return cls(raid_attr.name, raid_attr.instance_id,
+                   raid_attr.current_value, raid_attr.pending_value,
+                   raid_attr.read_only, raid_attr.fqdd, possible_values)
+
+    def validate(self, new_value):
+        """Validates new value"""
+
+        if str(new_value) not in self.possible_values:
+            msg = ("Attribute '%(attr)s' cannot be set to value '%(val)s'."
+                   " It must be in %(possible_values)r.") % {
+                   'attr': self.name,
+                   'val': new_value,
+                   'possible_values': self.possible_values}
+            return msg
+
+
+class RAIDStringAttribute(RAIDAttribute):
+    """String RAID attribute class"""
+
+    namespace = uris.DCIM_RAIDString
+
+    def __init__(self, name, instance_id, current_value, pending_value,
+                 read_only, fqdd, min_length, max_length):
+        """Creates RAIDStringAttribute object
+
+        :param name: name of the RAID attribute
+        :param instance_id: InstanceID of the RAID attribute
+        :param current_value:  list containing the current values of the
+                RAID attribute
+        :param pending_value: pending value of the RAID attribute, reflecting
+                an unprocessed change (eg. config job not completed)
+        :param read_only: indicates whether this RAID attribute can be changed
+        :param fqdd: Fully Qualified Device Description of the RAID
+                Attribute
+        :param min_length: minimum length of the string
+        :param max_length: maximum length of the string
+        """
+        super(RAIDStringAttribute, self).__init__(name, instance_id,
+                                                  current_value, pending_value,
+                                                  read_only, fqdd)
+        self.min_length = min_length
+        self.max_length = max_length
+
+    @classmethod
+    def parse(cls, raid_attr_xml):
+        """Parses XML and creates RAIDStringAttribute object"""
+
+        raid_attr = RAIDAttribute.parse(cls.namespace, raid_attr_xml)
+        min_length = int(utils.get_wsman_resource_attr(
+            raid_attr_xml, cls.namespace, 'MinLength'))
+        max_length = int(utils.get_wsman_resource_attr(
+            raid_attr_xml, cls.namespace, 'MaxLength'))
+
+        return cls(raid_attr.name, raid_attr.instance_id,
+                   raid_attr.current_value, raid_attr.pending_value,
+                   raid_attr.read_only, raid_attr.fqdd,
+                   min_length, max_length)
+
+
+class RAIDIntegerAttribute(RAIDAttribute):
+    """Integer RAID attribute class"""
+
+    namespace = uris.DCIM_RAIDInteger
+
+    def __init__(self, name, instance_id, current_value, pending_value,
+                 read_only, fqdd, lower_bound, upper_bound):
+        """Creates RAIDIntegerAttribute object
+
+        :param name: name of the RAID attribute
+        :param instance_id: InstanceID of the RAID attribute
+        :param current_value: list containing the current value of the
+                RAID attribute
+        :param pending_value: pending value of the RAID attribute,
+                reflecting an unprocessed change
+                (eg. config job not completed)
+        :param read_only: indicates whether this RAID attribute can be
+                changed
+        :param fqdd: Fully Qualified Device Description of the RAID
+                Attribute
+        :param lower_bound: minimum value for the RAID attribute
+        :param upper_bound: maximum value for the RAID attribute
+        """
+        super(RAIDIntegerAttribute, self).__init__(name, instance_id,
+                                                   current_value,
+                                                   pending_value,
+                                                   read_only, fqdd)
+        self.lower_bound = lower_bound
+        self.upper_bound = upper_bound
+
+    @classmethod
+    def parse(cls, raid_attr_xml):
+        """Parses XML and creates RAIDIntegerAttribute object"""
+
+        raid_attr = RAIDAttribute.parse(cls.namespace, raid_attr_xml)
+        lower_bound = utils.get_wsman_resource_attr(
+            raid_attr_xml, cls.namespace, 'LowerBound')
+        upper_bound = utils.get_wsman_resource_attr(
+            raid_attr_xml, cls.namespace, 'UpperBound')
+
+        if raid_attr.current_value:
+            raid_attr.current_value = int(raid_attr.current_value[0])
+        if raid_attr.pending_value:
+            raid_attr.pending_value = int(raid_attr.pending_value)
+
+        return cls(raid_attr.name, raid_attr.instance_id,
+                   raid_attr.current_value, raid_attr.pending_value,
+                   raid_attr.read_only, raid_attr.fqdd,
+                   int(lower_bound), int(upper_bound))
+
+    def validate(self, new_value):
+        """Validates new value"""
+
+        val = int(new_value)
+        if val < self.lower_bound or val > self.upper_bound:
+            msg = ('Attribute %(attr)s cannot be set to value %(val)d.'
+                   ' It must be between %(lower)d and %(upper)d.') % {
+                       'attr': self.name,
+                       'val': new_value,
+                       'lower': self.lower_bound,
+                       'upper': self.upper_bound}
+            return msg
+
+
 class RAIDManagement(object):
 
+    NAMESPACES = [(uris.DCIM_RAIDEnumeration, RAIDEnumerableAttribute),
+                  (uris.DCIM_RAIDString, RAIDStringAttribute),
+                  (uris.DCIM_RAIDInteger, RAIDIntegerAttribute)]
+
     def __init__(self, client):
         """Creates RAIDManagement object
 
@@ -108,6 +321,54 @@ class RAIDManagement(object):
         """
         self.client = client
 
+    def list_raid_settings(self):
+        """List the RAID configuration settings
+
+        :returns: a dictionary with the RAID settings using InstanceID as the
+                  key. The attributes are either RAIDEnumerableAttribute,
+                  RAIDStringAttribute objects.
+        :raises: WSManRequestFailure on request failures
+        :raises: WSManInvalidResponse when receiving invalid response
+        :raises: DRACOperationFailed on error reported back by the DRAC
+                interface
+        """
+
+        return utils.list_settings(self.client, self.NAMESPACES,
+                                   by_name=False)
+
+    def set_raid_settings(self, raid_fqdd, new_settings):
+        """Sets the RAID configuration
+
+        It sets the pending_value parameter for each of the attributes
+        passed in. For the values to be applied, a config job must
+        be created.
+        :param raid_fqdd: the FQDD of the RAID setting.
+        :param new_settings: a dictionary containing the proposed values, with
+                         each key being the name of attribute and the value
+                         being the proposed value.
+        :returns: a dictionary containing:
+                 - The is_commit_required key with a boolean value indicating
+                   whether a config job must be created for the values to be
+                   applied.
+                 - The is_reboot_required key with a RebootRequired enumerated
+                   value indicating whether the server must be rebooted for the
+                   values to be applied. Possible values are true and false.
+        :raises: WSManRequestFailure on request failures
+        :raises: WSManInvalidResponse when receiving invalid response
+        :raises: DRACOperationFailed on error reported back by the DRAC
+                interface
+        """
+
+        return utils.set_settings('RAID',
+                                  self.client,
+                                  self.NAMESPACES,
+                                  new_settings,
+                                  uris.DCIM_RAIDService,
+                                  "DCIM_RAIDService",
+                                  "DCIM:RAIDService",
+                                  raid_fqdd,
+                                  by_name=False)
+
     def list_raid_controllers(self):
         """Returns the list of RAID controllers
 
@@ -141,7 +402,7 @@ class RAIDManagement(object):
                                                'PrimaryStatus')],
             firmware_version=self._get_raid_controller_attr(
                 drac_controller, 'ControllerFirmwareVersion'),
-            bus=self._get_raid_controller_attr(drac_controller, 'Bus'),
+            bus=self._get_raid_controller_attr(drac_controller, 'Bus').upper(),
             supports_realtime=RAID_CONTROLLER_IS_REALTIME[
                 self._get_raid_controller_attr(
                     drac_controller, 'RealtimeCapability')])
@@ -266,6 +527,11 @@ class RAIDManagement(object):
                                                        uri)
         drac_bus_protocol = self._get_physical_disk_attr(drac_disk,
                                                          'BusProtocol', uri)
+        bus = self._get_physical_disk_attr(drac_disk,
+                                           'Bus', uri,  allow_missing=True)
+
+        if bus is not None:
+            bus = bus.upper()
 
         return PhysicalDisk(
             id=fqdd,
@@ -291,7 +557,8 @@ class RAIDManagement(object):
             device_protocol=self._get_physical_disk_attr(drac_disk,
                                                          'DeviceProtocol',
                                                          uri,
-                                                         allow_missing=True))
+                                                         allow_missing=True),
+            bus=bus)
 
     def _get_physical_disk_attr(self, drac_disk, attr_name, uri,
                                 allow_missing=False):
